from .Button import Button
from .Slider import Slider
from .ColorPicker import ColorPicker
from .DatePicker import DatePicker
from .Progress import Progress
from .Radio import Radio
from .IconClipboard import IconClipboard
from .Switch import Switch
from .SliderFloat import SliderFloat
from .Label import Label
from .DayCalendar import DayCalendar
from .Toggle import Toggle
from .MultiSwitch import MultiSwitch
from .Tabs import Tabs
from .TextList import TextList
from .RangeSlider import RangeSlider
# from .Menu import Menu

from .deprecated import button
from .deprecated import slider
from .deprecated import colorPicker
from .deprecated import datePicker
from .deprecated import progress
from .deprecated import radio
from .deprecated import switch
from .deprecated import sliderFloat
from .deprecated import label
from .deprecated import dayCalendar
from .deprecated import toggle
from .deprecated import multiSwitch
from .deprecated import tabs
from .deprecated import textlist
from .deprecated import rangeSlider
# from .deprecated import menu